/* 
 * (c) Copyright IBM Corp. 2005, 2006. All Rights Reserved. 
 * Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms and conditions in the IBM International Program License Agreement. 
 */
package export;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;


import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.URI;
import org.eclipse.hyades.models.common.common.impl.CMNAnnotationImpl;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;

import com.ibm.rational.test.mt.exporter.interfaces.IMTAExportFormatter;
import com.ibm.rational.test.mt.interfaces.ILogDocument;
import com.ibm.rational.test.mt.interfaces.ILogEvent;

/**
 * HtmlFormatter
 *
 * This class is an implementation of IMTAExportFormatter interface and it provides functionality
 * to export the execution results into a HTML format and stores them to a file.
 */
public class HtmlFormatter implements IMTAExportFormatter
{
	/**
	 * Data source for export - the test log.
	 */
	private ILogDocument m_logDoc;
	
	/**
	 * Path where the exported results will be stored.
	 */
	private String m_destFile;
	
	/**
	 * The most recent error that has occurred.
	 */
	private String m_lastErr = null;
	
	static final String FILE_EXTENSION = ".html";
	
	/**
	 * Empty Constructor
	 *
	 */
	public HtmlFormatter( )
	{
		this( null, null );
	}
	
	public HtmlFormatter( ILogDocument logDoc, String destFile )
	{
		m_logDoc = logDoc;
		m_destFile = destFile;
	}
	
	/**
	 * Set the LogDocument which is the test log.
	 * 
	 * @param logDoc
	 */
	public void setLogDocument( ILogDocument logDoc )
	{
		m_logDoc = logDoc;
	}
	
	/**
	 * Set the destination file where the export results will be written.
	 * @param destFile
	 */
	public void setDestinationFile( String destFile )
	{
		m_destFile = destFile;
	}
	
	/**
	 * Return the full destination file name.
	 * @param bOpen
	 */
	public String getDestinationFile()
	{
		String fileName = formatFileExtension( m_destFile );
		return fileName;
	}
	
	/**
	 * Gets the last error that occurred.
	 * This will be displayed to the users upon the completion of the Runnable Task.
	 * 
	 * @return null, if there is no error.
	 */
	public String getLastError( )
	{
		return m_lastErr;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.operation.IRunnableWithProgress#run(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void run(IProgressMonitor monitor) throws InvocationTargetException,	InterruptedException 
	{
		if (( m_logDoc == null ) || ( m_destFile == null ))
			return;
		
		monitor.beginTask( "Exporting execution results...", IProgressMonitor.UNKNOWN );
		
		ArrayList list = m_logDoc.getRootEvents( );
		Iterator it = list.iterator( );
		StringBuffer buffer = new StringBuffer();
		
		buffer.append("<HTML>\r\n");
		buffer.append("<HEAD>\r\n");
		buffer.append("<META HTTP-EQUIV=\"charset\" CONTENT=\"utf-8\">\r\n");
		buffer.append("<TITLE>Test Log for "+ m_logDoc.getScriptFileName()+"</TITLE>\r\n");
		addStyleSheets(buffer);
		buffer.append("</HEAD>\r\n");
		buffer.append("<BODY>\r\n");
		buffer.append("<TABLE WIDTH=\"100%\" border=\"1\" rules=\"rows\" summary=\"Log Events\">\r\n");
		buffer.append("<CAPTION>Test Log: "+m_logDoc.getFileName()+"</CAPTION>\r\n");
		
		String fileName = formatFileExtension( m_destFile );
		try
		{
			// write out all events in the log
			while ( it.hasNext( ) )
			{
				ILogEvent e = (ILogEvent)it.next( );
				formatAndExportEvent( e, buffer, monitor, 0 );
			}
		
			buffer.append("</TABLE>\r\n");
			buffer.append("</BODY>\r\n");
			buffer.append("</HTML>\r\n");
			
			// Open filestream and write to it.			
			monitor.setTaskName( "Writing export results to " + fileName );
			FileWriter writer = new FileWriter( fileName, true );
			writer.write( buffer.toString( ) );
			writer.close( );
		}
		catch (Exception ex)
		{
			String title = "Sample - RMT HTML Export";
			String err = "Error writing exported test log(s) to file \"" + fileName + "\".\nReason:\n" + ex.getMessage( );
			MessageDialog.openError( new Display().getActiveShell(), title, err );
			m_lastErr = err;
		}
		monitor.done( );
	}

	/**
	 * Recursively formats and exports the LogEvent into a IOStream.
	 * @param event
	 * @param buffer
	 * @param monitor
	 * @param level
	 */
	private void formatAndExportEvent( ILogEvent event, StringBuffer buffer, IProgressMonitor monitor, int level )
	{
		monitor.setTaskName( "Exporting Event " + event.getName() );
		if ( level > 0 )  //Don't export root node
		{
			buffer.append("<TR>\r\n");
			
			// verdict
			String sResult = event.getProperty("Result");
			if ((sResult != null) && (sResult.length() > 0))
			    buffer.append("<TD CLASS=\""+event.getResult()+"\">"+sResult+"</TD>\r\n");
			else
			    buffer.append("<TD>&nbsp;</TD>\r\n");
			
			// time
			Date date = new Date(event.getTimestamp());
			SimpleDateFormat format = new SimpleDateFormat("yyyy.MM.dd  hh:mm:ss:SS"); 
			String sDate = format.format(date);
			buffer.append("<TD CLASS=\"time\">"+sDate+"</TD>\r\n");
			
			// name
			buffer.append("<TD CLASS=\"name\">"+event.getName()+"</TD>\r\n" );
			
			buffer.append("</TR>\r\n");

			// More details about the log event
			buffer.append("<TR><TD COLSPAN=3><UL>\r\n");
			
			// file containing the test step (interesting when a reused step)
			File file = new File(m_logDoc.getFileName());
			buffer.append("<LI><I>file name = </I>"+file.getName()+"</LI>\r\n");
			
			// step type
			buffer.append("<LI><I>type = </I>"+convertTypeToRmtType(event.getType())+"</LI>\r\n");
			
			// description
			buffer.append("<LI><I>description = </I>"+event.getText()+"</LI>\r\n");
			
			// string properties
			Hashtable properties = event.getProperties();
			for (Enumeration e = properties.keys(); e.hasMoreElements();)
			{
				String propName = (String)e.nextElement();
				// ignore values that have been output already
				if (!propName.equalsIgnoreCase("Type") && !propName.equalsIgnoreCase("Result") &&
				    !propName.equalsIgnoreCase(".Execution Attachment"))
				{
					Object propValue = properties.get(propName);
					if (propValue instanceof String)
					{
						buffer.append("<LI><I>"+propName+" = </I>"+propValue.toString()+"</LI>\r\n");
					}
				}
			}
			
			// attachments
			ArrayList attachments = event.getAttachments();
			if ( attachments != null )
			{
				for (int i=0; i<attachments.size(); i++)
				{
					Object obj = attachments.get(i);
					CMNAnnotationImpl attachment = (CMNAnnotationImpl)obj;
					try {
						if(attachment.isFileAnnotation())
						{
							URI uri = attachment.getFileAnnotation();
							String path = uri.toFileString();
							//TODO: path is a TPTP temporary location.  This location works
							//for an example but for production code you should copy the file
							//to a different location (probably the same directory where the
							//exported file is being written) and fix up the "path" in the html
							//below to reference the new location.
							if (isGraphic(path))
							{
								buffer.append("<LI><I>attachment = </I><IMG SRC=\""+path+"\"></LI>\r\n");
							}
							else
							{
								// Handle non-graphic attachments here
							}
						}
					} catch (IOException e)	{}
				}
			}
			
			buffer.append("</UL>\r\n");
			buffer.append("</TD></TR>\r\n");
		}
		
		if ( event.hasChildren() )		// group
		{
			ArrayList children = event.getChildren();
			Iterator it = children.iterator();
			while ( it.hasNext() )
				formatAndExportEvent( (ILogEvent)it.next(), buffer, monitor, level+1 );
		}
	}
	
	private String formatFileExtension( String file )
	{
		// If the file doesn't have any extension, or the extension is
		// different from html, this will overwrite or append it.
		String fileName = file;
		int pos = file.lastIndexOf( "." );
		if ( pos != -1 )
			fileName = fileName.substring( 0, pos );
		fileName += FILE_EXTENSION;
		
		return fileName;
	}

	/**
	 * Add style sheet(s) to the output buffer
	 * @param buffer
	 */
	private void addStyleSheets(StringBuffer buffer)
	{
		buffer.append("<STYLE TYPE=\"text/css\">\r\n");
		buffer.append("<!--\r\n");
		buffer.append("table caption {color:darkblue; font-size:x-large; font-weight:bold}");
		buffer.append("td {align:left; font-weight:bold}\r\n");
		buffer.append("td.pass {color:darkgreen; width:10%}\r\n");
		buffer.append("td.fail {color:darkred; width:10%}\r\n");
		buffer.append("td.error {color:darkgoldenrod; width:10%}\r\n");
		buffer.append("td.inconclusive {color:darkcyan; width:10%}\r\n");
		buffer.append("td.time {width:30%; font-weight:normal}\r\n");
		buffer.append("td.name {align:left; font-weight:bold}\r\n");
		buffer.append("li {font-weight: normal}\r\n");
		buffer.append("-->\r\n");
		buffer.append("</STYLE>\r\n");
	}
	
	/**
	 * Return the RMT step type given the TPTP step type.
	 * @return
	 */
	private String convertTypeToRmtType( String tptpStepType )
	{
		if (tptpStepType.equalsIgnoreCase("ManualStep") )
			return "Step";
		else if (tptpStepType.equalsIgnoreCase("ManualFolder"))
			return "Group";
		else if (tptpStepType.equalsIgnoreCase("ManualVP"))
			return "Verification Point";
		else if (tptpStepType.equalsIgnoreCase("ManualRP"))
			return "Reporting Point";
		
		return "Unknown Type";	// should never happen
	}
	
	/**
	 * Return whether filePath specifies a graphic, or not
	 * @param filePath
	 * @return
	 */
	private boolean isGraphic( String filePath )
	{
		String extension = null;
		int index = filePath.lastIndexOf(".");
		if (index >= 0)
			extension = filePath.substring(index+1);
		
		return (extension.equalsIgnoreCase("gif") || 
		        extension.equalsIgnoreCase("jpg"));
	}
}
